from copy import deepcopy, copy
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
from django.contrib.contenttypes.admin import GenericTabularInline, GenericStackedInline
from django.forms import ModelForm, NumberInput
from django.db import models


class SortableModelAdminBase(object):
    """
    Base class for SortableTabularInline and SortableModelAdmin
    """
    sortable = 'order'

    class Media:
        js = ('admin/js/jquery.init.js', 'suit/js/suit.sortables.js',)


class SortableListForm(ModelForm):
    """
    Just Meta holder class
    """

    class Meta:
        widgets = {
            'order': NumberInput(
                attrs={'class': 'd-none suit-sortable'})
        }


class SortableChangeList(ChangeList):
    """
    Class that forces ordering by sortable param only
    """

    def get_ordering(self, request, queryset):
        if self.model_admin.sortable_is_enabled():
            return [self.model_admin.sortable, '-' + self.model._meta.pk.name]
        return super(SortableChangeList, self).get_ordering(request, queryset)


class SortableTabularInlineBase(SortableModelAdminBase):
    """
    Sortable tabular inline
    """

    def __init__(self, *args, **kwargs):
        super(SortableTabularInlineBase, self).__init__(*args, **kwargs)

        self.ordering = (self.sortable,)
        self.fields = self.fields or []
        if self.fields and self.sortable not in self.fields:
            self.fields = list(self.fields) + [self.sortable]

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == self.sortable:
            kwargs['widget'] = SortableListForm.Meta.widgets['order']
        return super(SortableTabularInlineBase, self).formfield_for_dbfield(
            db_field, **kwargs)


class SortableTabularInline(SortableTabularInlineBase, admin.TabularInline):
    pass


class SortableGenericTabularInline(SortableTabularInlineBase,
                                   GenericTabularInline):
    pass


class SortableStackedInlineBase(SortableModelAdminBase):
    """
    Sortable stacked inline
    """

    def __init__(self, *args, **kwargs):
        super(SortableStackedInlineBase, self).__init__(*args, **kwargs)
        self.ordering = (self.sortable,)

    def get_fieldsets(self, *args, **kwargs):
        """
        Iterate all fieldsets and make sure sortable is in the first fieldset
        Remove sortable from every other fieldset, if by some reason someone
        has added it
        """
        fieldsets = super(SortableStackedInlineBase, self).get_fieldsets(*args, **kwargs)

        sortable_added = False
        for fieldset in fieldsets:
            for line in fieldset:
                if not line or not isinstance(line, dict):
                    continue

                fields = line.get('fields')
                if self.sortable in fields:
                    fields.remove(self.sortable)

                # Add sortable field always as first
                if not sortable_added:
                    fields.insert(0, self.sortable)
                    sortable_added = True
                    break

        return fieldsets

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == self.sortable:
            kwargs['widget'] = deepcopy(SortableListForm.Meta.widgets['order'])
            kwargs['widget'].attrs['class'] += ' suit-sortable-stacked'
            kwargs['widget'].attrs['rowclass'] = ' suit-sortable-stacked-row'
        return super(SortableStackedInlineBase, self).formfield_for_dbfield(db_field, **kwargs)


class SortableStackedInline(SortableStackedInlineBase, admin.StackedInline):
    pass


class SortableGenericStackedInline(SortableStackedInlineBase,
                                   GenericStackedInline):
    pass


class SortableModelAdmin(SortableModelAdminBase, admin.ModelAdmin):
    """
    Sortable change list
    """

    def __init__(self, *args, **kwargs):
        super(SortableModelAdmin, self).__init__(*args, **kwargs)

        # Keep originals for restore
        self._original_ordering = copy(self.ordering)
        self._original_list_display = copy(self.list_display)
        self._original_list_editable = copy(self.list_editable)
        self._original_exclude = copy(self.exclude)
        self._original_list_per_page = self.list_per_page

        self.enable_sortable()

    def merge_form_meta(self, form):
        """
        Prepare Meta class with order field widget
        """
        if not getattr(form, 'Meta', None):
            form.Meta = SortableListForm.Meta
        if not getattr(form.Meta, 'widgets', None):
            form.Meta.widgets = {}
        form.Meta.widgets[self.sortable] = SortableListForm.Meta.widgets[
            'order']

    def get_changelist_form(self, request, **kwargs):
        form = super(SortableModelAdmin, self).get_changelist_form(request,
                                                                   **kwargs)
        self.merge_form_meta(form)
        return form

    def get_changelist(self, request, **kwargs):
        return SortableChangeList

    def enable_sortable(self):
        self.list_per_page = 500
        self.ordering = (self.sortable,)
        if self.list_display and self.sortable not in self.list_display:
            self.list_display = list(self.list_display) + [self.sortable]

        self.list_editable = self.list_editable or []
        if self.sortable not in self.list_editable:
            self.list_editable = list(self.list_editable) + [self.sortable]

        self.exclude = self.exclude or []
        if self.sortable not in self.exclude:
            self.exclude = list(self.exclude) + [self.sortable]

    def disable_sortable(self):
        if not self.sortable_is_enabled():
            return
        self.ordering = self._original_ordering
        self.list_display = self._original_list_display
        self.list_editable = self._original_list_editable
        self.exclude = self._original_exclude
        self.list_per_page = self._original_list_per_page

    def sortable_is_enabled(self):
        return self.list_display and self.sortable in self.list_display

    def save_model(self, request, obj, form, change):
        if not obj.pk:
            max_order = obj.__class__.objects.aggregate(
                models.Max(self.sortable))
            try:
                next_order = max_order['%s__max' % self.sortable] + 1
            except TypeError:
                next_order = 1
            setattr(obj, self.sortable, next_order)
        super(SortableModelAdmin, self).save_model(request, obj, form, change)



import django
from django import template
from django.utils.safestring import mark_safe
from suit import config
from suit.config import get_config

register = template.Library()

if django.VERSION < (1, 9):
    simple_tag = register.assignment_tag
else:
    simple_tag = register.simple_tag

def get_form_size(fieldset):
    form_size_by_config = get_config('form_size')

    # Fallback to model admin definition
    form_size_by_model_admin = getattr(fieldset.model_admin, 'suit_form_size', {})

    form_size = {}
    form_size.update(form_size_by_config)
    form_size.update(form_size_by_model_admin)

    return form_size


def get_form_class(field, fieldset, idx):
    field_class, extra_class = [], ''
    form_size = get_form_size(fieldset)

    if not form_size:
        raise Exception('"form_size" parameter must be set in Django Suit config')

    # Try field config first
    if not field_class:
        form_size_fields = form_size.get('fields')
        if form_size_fields:
            field_name = None
            if hasattr(field, 'name'):
                field_name = field.name
            elif isinstance(field, dict) and 'name' in field:
                # field may be a dict as well (for stacked inlines)
                field_name = field['name']
            if field_name:
                field_class = form_size_fields.get(field_name)

    # Add widgets CSS class
    if idx == 1:
        extra_class = ' %s' % suit_form_field_widget_class(field)

    # Try widgets config
    widget_class_name = get_field_widget_class_name(field)
    if not field_class and widget_class_name:
        form_size_widgets = form_size.get('widgets')
        if form_size_widgets:
            field_class = form_size_widgets.get(widget_class_name)

    # Try fieldset config
    if not field_class:
        form_size_fieldset = form_size.get('fieldsets')
        if form_size_fieldset:
            field_class = form_size_fieldset.get(fieldset.name)

    # Fallback to default
    if not field_class:
        field_class = form_size.get('default')

    assert isinstance(field_class, (tuple, list)) and len(field_class) == 2, \
        u'Django Suit form_size definition must be list or tuple containing two string items. ' \
        u'You have: "%s" (%s)' % (field_class, field_class.__class__)

    return field_class[idx] + extra_class


def get_field_widget_class_name(field):
    try:
        widget_class_name = field.field.widget.__class__.__name__
        return widget_class_name
    except AttributeError:
        pass


@register.filter
def suit_form_label_class(field, fieldset):
    """
    Get CSS class for form-row label column
    """
    return get_form_class(field, fieldset, 0)


@register.filter
def suit_form_field_class(field, fieldset):
    """
    Get CSS class for form-row field column
    """
    return get_form_class(field, fieldset, 1)


@register.filter
def suit_form_field_widget_class(field):
    """
    Get CSS class for field by widget name, for easier styling
    """
    widget_class_name = get_field_widget_class_name(field)
    if widget_class_name:
        return 'widget-%s' % widget_class_name
    return ''


@simple_tag(takes_context=True)
def suit_form_conf(context, param_name, inline_admin_formset=None):
    """
    Get form config param
    """
    if inline_admin_formset:
        model_admin = inline_admin_formset.opts
    else:
        model_admin = context['adminform'].model_admin
    param_by_model_admin = getattr(model_admin, 'suit_%s' % param_name, None)
    if param_by_model_admin is not None:
        return param_by_model_admin
    return get_config(param_name, context['request'])


@register.filter
def suit_form_field_placeholder(field, placeholder):
    """
    Get CSS class for field by widget name, for easier styling
    """
    field.field.widget.attrs['placeholder'] = placeholder
    return field



from django.utils.translation import gettext_lazy as _
from django.contrib.admin import FieldListFilter


class IsNullFieldListFilter(FieldListFilter):
    notnull_label = _('Is present')
    isnull_label = _('Is Null')

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg = '%s__isnull' % field_path
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)
        super(IsNullFieldListFilter, self).__init__(field,
                                                    request, params, model,
                                                    model_admin, field_path)

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def choices(self, cl):
        for lookup, title in (
                (None, _('All')),
                ('False', self.notnull_label),
                ('True', self.isnull_label),
        ):
            yield {
                'selected': self.lookup_val == lookup,
                'query_string': cl.get_query_string({
                    self.lookup_kwarg: lookup,
                }),
                'display': title,
            }



import django
from django import template
from django.utils.safestring import mark_safe
from suit import config

register = template.Library()

if django.VERSION < (1, 9):
    simple_tag = register.assignment_tag
else:
    simple_tag = register.simple_tag


@register.filter(name='suit_conf')
def suit_conf(name, request):
    value = config.get_config(name, request)
    return mark_safe(value) if isinstance(value, str) else value


@register.filter(name='suit_body_class')
def suit_body_class(value, request):
    css_classes = []
    config_vars_to_add = ['toggle_changelist_top_actions', 'form_submit_on_right', 'layout']
    for each in config_vars_to_add:
        suit_conf_param = getattr(config.get_config(None, request), each, None)
        if suit_conf_param:
            value = each if isinstance(suit_conf_param, bool) \
                else '_'.join((each, suit_conf_param))
            css_classes.append('suit_%s' % value)
    return ' '.join(css_classes)


@simple_tag
def suit_conf_value(name, model_admin=None):
    if model_admin:
        value_by_ma = getattr(model_admin, 'suit_%s' % name.lower(), None)
        if value_by_ma in ('center', 'right'):
            config.set_config_value(name, value_by_ma)
        else:
            config.reset_config_value(name)
    return suit_conf(name)



from django.apps import apps
from django.template import VariableDoesNotExist
from suit.apps import DjangoSuitConfig


def get_config_instance(app_name=None):
    """
    :rtype: DjangoSuitConfig()
    """
    try:
        config = apps.get_app_config(app_name or 'suit')
        if isinstance(config, DjangoSuitConfig):
            return config
    except LookupError:
        pass
    return apps.get_app_config('suit')


#: :type: DjangoSuitConfig()
suit_config_cls = DjangoSuitConfig


def get_config(param=None, request=None):
    suit_config = get_config_instance(get_current_app(request) if request else None)

    # Allow overriding suit config by request
    # Used only for demo purposes
    req_key = '__suit_config_by_request'
    if request and not hasattr(req_key, req_key):
        setattr(request, req_key, True)
        for k, v in request.GET.items():
            if k.startswith('__suit_'):
                setattr(suit_config, k[7:], v)

    if param:
        value = getattr(suit_config, param, None)
        if value is None:
            value = getattr(suit_config_cls, param, None)
        return value

    return suit_config


def get_current_app(request):
    try:
        return request.current_app
    except (VariableDoesNotExist, AttributeError):
        pass
    return None


def set_config_value(name, value):
    config = get_config()
    # Store previous value to reset later if needed
    prev_value_key = '_%s' % name
    if not hasattr(config, prev_value_key):
        setattr(config, prev_value_key, getattr(config, name))
        setattr(config, name, value)


def reset_config_value(name):
    config = get_config()
    prev_value_key = '_%s' % name
    if hasattr(config, prev_value_key):
        setattr(config, name, getattr(config, prev_value_key))
        del config.__dict__[prev_value_key]
